# $Id: EximPerl.pm,v 1.18.4.1 2002/04/30 14:21:48 bengen Exp $

#
# MTA module for Exim+Embedded Perl setup
#

package AMAVIS::MTA::EximPerl;
use strict;
use vars qw($VERSION);
$VERSION='0.1';

use AMAVIS;
use AMAVIS::Logging;
use IO::File;
use File::Path;

# For "freezing"
use File::Copy;
use Sys::Hostname;

use vars qw(
	    $cfg_exim_binary
	    $cfg_exim_args
	    $cfg_problem_dir
	   );

sub init {
  my $self = shift;
  my $args = shift;
  $cfg_exim_binary = $AMAVIS::cfg->val('Exim', 'exim');
  $cfg_exim_args = $AMAVIS::cfg->val('Exim', 'args');
  if ($cfg_exim_binary eq '') {
    writelog($args,LOG_CRIT, "Location of exim binary not specified");
    return 0;
  }
  if (! -x $cfg_exim_binary) {
    writelog($args,LOG_CRIT, "$cfg_exim_binary not executable");
    return 0;
  }

  $cfg_problem_dir = $AMAVIS::cfg->val('Exim', 'problem dir');
  if ($cfg_problem_dir eq '') {
    writelog($args,LOG_CRIT, "Exim problem directory not specified");
    return 0;
  }
  if (! ( -x $cfg_problem_dir && -d $cfg_problem_dir &&
	  -w $cfg_problem_dir) ) {
    writelog($args,LOG_CRIT, "Directory $cfg_problem_dir not usable");
    return 0;
  }

  writelog($args,LOG_DEBUG,__PACKAGE__." initialized.");
  # writelog($args,LOG_INFO,"Working on Message-ID ".$$args{'eximperl_mid'});
  # Return successfully
  return 1;
}

# Create directory for unpacking and inspecting message
sub create_unpack_dir() {
  my $directory = create_tmp_dir($AMAVIS::cfg->val('paths', 'unpack').'amavis-unpack-');
  return $directory;
}

# Create temp directory. Try creating $prefix[date]-[pid] 
# Return created dir or undef if unsuccessful after 10 tries
sub create_tmp_dir( $ ) {
  my $prefix = shift;
  my $i = 0;
  while (1) {
    my $tmpdir = sprintf("$prefix%.8x-%.4x",time,$$);
    unless (mkpath ($tmpdir, 0, 0770)) {
      if (++$i > 10) {
	return undef;
      }
      else {
	next;
      }
    }
    else {
      return $tmpdir;
    }
  }
}

# Create temp dir and write mail
sub get_directory($) {
  my $self = shift;
  my $args = shift;

  writelog($args,LOG_DEBUG, "Working with Exim Message ID $$args{eximperl_mid}");
  my $log_sender = ($$args{'sender'} or '<>');
  writelog($args,LOG_DEBUG, "Sender: $log_sender");
  writelog($args,LOG_DEBUG, "Recipient(s): ".join (' ',@{$$args{'recipients'}}));

  # Create temp directory
  my $directory = create_unpack_dir();
  mkdir "$directory/parts", 0777;

  # Open message file that is later going to be disected and scanned
  my $output = IO::File->new("+>$directory/email.txt");

  # Get header file.
  my $exim = IO::File->new("$cfg_exim_binary -Mvh ".$$args{eximperl_mid}."|") 
    or return undef;

  # Skip over Exim's information
  while (<$exim>) {
    last if /^\s*$/;
  }

  # Read header
  $$args{'headers'}.='';
  while (<$exim>) {
    s/^[0-9]{3,}. //;
    print $output $_;
    $$args{'headers'}.=$_;
  }
  $exim->close();

  # Insert newline to separate header lines and body
  print $output "\n";

  # Get body file
  $exim = IO::File->new("$cfg_exim_binary -Mvb ".$$args{eximperl_mid}."|") 
    or return undef;

  # Read body
  <$exim>; while (<$exim>) {
    print $output $_;
  }
  $exim->close();

  # Message file has been written, reset file pointer and put it into
  # the record.
  $output->seek(0,0);
  $$args{'filehandle'} = $output;
  $$args{'directory'} = $directory;

  # Return successfully
  return 1;
}

sub accept_message($) {
  my $self = shift;
  my $args = shift;
  writelog($args,LOG_INFO, __PACKAGE__.": Accepting message ".$$args{'eximperl_mid'});
  $$args{'eximperl_returncode'} = 'accept';
  $$args{'status'} = 'accept';

  # Return successfully
  return 1;
}

sub drop_message($) {
  my $self = shift;
  my $args = shift;
  writelog($args,LOG_INFO, __PACKAGE__.": Dropping message ".$$args{'eximperl_mid'});
  $$args{'eximperl_returncode'} = 'drop';

  # Return successfully
  return 1;
}

sub freeze_message( $ ) {
  my $self = shift;
  my $args = shift;
  writelog($args,LOG_INFO, __PACKAGE__.": Freezing message ".$$args{'eximperl_mid'});

#  $$args{'eximperl_returncode'} = 'freeze';
#   $main::returntext = 
#     "AMAVIS had a problem with this message. See logs for details";

  # Instead of freezing, drop the message, but store a copy in
  # $cfg_problem_dir -- analogous to AMAVIS::quarantine_message()

  writelog($args,LOG_DEBUG, "Putting message into problem dir");
  if (copy("$$args{'directory'}/email.txt",
	   "$cfg_problem_dir/$$args{'eximperl_mid'}.msg")) {
    $$args{'eximperl_returncode'} = 'drop';
    # write part of logfile to quarantine dir.
    my $logfile = IO::File->new(">$cfg_problem_dir/".
				"$$args{'eximperl_mid'}.log");
    unless ((defined $logfile) && 
	    ($logfile->print($$args{'log'})) &&
	    ($logfile->close())) {
      writelog($args,LOG_CRIT, "Writing log into problem dir failed");
    }
    my $hostname=hostname();
    my $error_address=$AMAVIS::cfg->val('Notify', 'admin');
    my $msg=<<"EOF";
Subject: Message put in AMaViS-Exim problem directory

Message $$args{'eximperl_mid'}
has been put into AMaViS-Exim problem directory as
$hostname:$cfg_problem_dir/$$args{'eximperl_mid'}.msg.

The corresponding logfile has been written to
$hostname:$cfg_problem_dir/$$args{'eximperl_mid'}.log.

EOF
    $self->send_message($args, $msg, $error_address, $error_address);
  }
  else {
    writelog($args,LOG_ERR, "Unable to put message into problem dir $cfg_problem_dir. Freezing message.");
    $$args{'eximperl_returncode'} = 'freeze';
  }

  # Return successfully
  return 1;
}

sub send_message($$) {
  my $self = shift;
  my $args = shift;
  my $message = shift;
  my $sender = shift;
  my @recipients = @_;
  writelog($args,LOG_DEBUG, __PACKAGE__.": Sending mail from $sender to ".
	   join(', ',@recipients));

  my @exim_args;

  push @exim_args, split(/\s+/,$cfg_exim_args);
  push @exim_args, $sender;
  push @exim_args, @recipients;

  writelog($args,LOG_DEBUG, __PACKAGE__.": Running $cfg_exim_binary ".
	   join(' ',@exim_args));

  open(MAIL, "|-") || exec($cfg_exim_binary, @exim_args);
  print MAIL $message;
  close(MAIL);

  if ($? != 0) {
    writelog($args,LOG_ERR,
	     __PACKAGE__.": $cfg_exim_binary @exim_args exited with ".($?>>8));
  }

  # Return successfully
  return 1;
}

1;
